module Controle(
//entrada no Controle
input clock,
input reset,
input overflow,
input [5:0] opcode,
input [5:0] funct,
input [31:0] memory,
//input nop,
input DivOut,
//Registradores
output reg LoadA,
output reg LoadB,
output reg EPCWrite,
output reg MDRLoad,
output reg PCWrite,
output reg Load_Hi,
output reg Load_Lo,
output reg Load_ALUOut,
output reg Load_shift_Out,
//mux
output reg StoreW,
output reg LoadW,
output reg SllorSlv,
output reg HiOrLo,
output reg ALUSourceA,
output reg [2:0] ALUSourceB,
output reg [1:0] MemoryAdress,
output reg [2:0] WriteData,
output reg [1:0] RegDest,
output reg [2:0] PCSource,
output reg Shift_ShamtOrRt,
output reg [1:0]BneOrBeq,
//componentes
output reg LoadHorB,
output reg StoreHorB,
output reg MemoryWriteOrRead,
output reg [2:0] Shift,
output reg RegWrite,
output reg [2:0] ALUControl,
output reg IRWrite,
output reg DivIn,

//estado
output wire [7:0] est

);

reg [7:0] estado;
assign est = estado;

parameter
busca_Instrucao = 1,
decodifica_Instrucao = 2,
exception = 3,
escreve_Rd = 4,
carrega_Shift = 5,
estado_overflow = 6,
escreve_EPC = 7,
escreve_Ra = 8,
load_MDR_storeb = 9,
escreve_IR = 10,
breaks = 11,
div = 12,
storeWord = 13,
storeByte = 14,
storeHalf = 15,
loadWord = 16,
loadByte = 17,
loadHalf = 18,
wait_Memory_StoreH = 19,
wait_Memory_StoreB = 20,
calculo_Endereco = 21,
jal = 22,
j = 23,
rte = 24,
mflo = 25,
mfhi = 26,
jr = 27,
add = 28,
addiu = 29,
addi = 30,
estado_and = 31,
lui = 32,
slt = 33,
slti = 34,
sll = 35,
sllv = 36,
sra = 37,
srav = 38,
srl = 39,
bne = 40,
beq = 41,
sub = 42,
load_MDR_storeh = 43,
opcode_inexistente = 44,
wait_memory = 45,
wait_overflow =46,
escreve_Shift = 48,
calc_end = 49,
estado_compara = 50,
escreve_PC = 51,
wait_Store = 52,
ler_memoria_loadb=53,
ler_memoria_loadh=54,
ler_memoria_loadw=55,
ler_memoria_storeb=56,
ler_memoria_storeh=57,
wait_memory_loadW = 58,
wait_memory_loadB = 59,
wait_memory_loadH = 60,
load_MDR_lh = 61,
load_MDR_lw = 62,
load_MDR_lb = 63,
carrega_shiftout = 64,
wait3 = 65,
wait4 = 66,
wait_overflow2 = 67,
escreve_Rt = 68,
wait5 = 69;

always @(posedge clock) 
begin

	LoadA = 1'b0;
	LoadB =  1'b0;
	PCWrite = 1'b0;
	EPCWrite = 1'b0;
	RegWrite = 1'b0;
	MemoryWriteOrRead = 1'b0;
	DivIn = 1'b0;
	IRWrite = 1'b0;
	Load_shift_Out = 1'b0;
	Load_ALUOut  = 1'b0;
	MDRLoad = 1'b0;
	BneOrBeq = 2'b11;
	
	if (reset == 1) begin
	
		RegDest = 2'b10;
		WriteData = 3'b110;
		RegWrite = 1'b1;
		estado <= busca_Instrucao;
		
	end
	
	case (estado)

		// BUSCA INSTRUO

		busca_Instrucao: begin 
		
			MemoryAdress = 2'b00; 
			MemoryWriteOrRead = 1'b0; 
			ALUSourceA = 1'b0;
			ALUSourceB = 3'b001;
			ALUControl = 3'b001;
			PCSource = 3'b100;
			PCWrite = 1'b1; 	
							
			estado <= wait_memory;
		
		end

		// WAIT MEMORY
	
		wait_memory: begin
			 
			estado <= escreve_IR;

		end

		// ESCREVE IR 
	
		escreve_IR: begin	
					
			IRWrite = 1'b1;
			estado <= wait3;
						
		end
		
		wait3:begin
	
		estado <= wait4;
		end 
	
		wait4:begin 
			LoadA = 1'b1;
			LoadB = 1'b1;
			
			if ( opcode == 6'h0 && funct == 6'h0 || opcode == 6'h0 &&  funct == 6'h3) begin
				SllorSlv = 1'b1;
			end
			else begin
				SllorSlv = 1'b0;
			end
	
			estado<= decodifica_Instrucao;
		end 
		// DECODIFICA INSTRUO

		decodifica_Instrucao: begin
			
		
				
			ALUSourceA = 1'b0;
			ALUSourceB = 3'b011;
			ALUControl = 3'b001;			
			Load_ALUOut  = 1'b1;
			
			case(opcode) 
		
				6'h0:  
					case(funct)	
				
						6'h20: 		estado <= add;

						6'h24:		estado <= estado_and;
				
						6'h1a:  	estado <= div;
	
						6'h8:  		estado <= jr;
	
						6'h10: 		estado <= mfhi;

						6'h12:  	estado <= mflo;

						6'h00:  begin 
								if(memory == 32'b0) begin                 //possvel NOP		
									estado <= busca_Instrucao;
									end	else begin
								estado <= carrega_Shift;
								end
								end
						6'h04:		estado <= carrega_Shift;

						6'h2a: 		estado <= slt;
	
						6'h03: 		estado <= carrega_Shift;
	
						6'h07:		estado <= carrega_Shift;

						6'h02:		estado <= carrega_Shift;

						6'h22: 		estado <= sub;
	
						6'hd:		estado <= breaks;
												  	
						
					endcase	
				6'h10:	estado <= rte;
				
				6'h8:   estado <= addi;

				6'h9:   estado <= addiu;

				6'h4:   estado <= estado_compara;

				6'h5:   estado <= estado_compara;
				
				6'h20:   estado <= calculo_Endereco;

				6'h21:   estado <= calculo_Endereco;

				6'hf:   estado <= lui;

				6'h23:   estado <= calculo_Endereco;

				6'h28:   estado <= calculo_Endereco;
 
				6'h29:   estado <= calculo_Endereco;

				6'ha:   estado <= slti;
 
				6'h2b:   estado <= calculo_Endereco;

				6'h2:   estado <= j;

				6'h3:   estado <= escreve_Ra;
				
				default: estado <= opcode_inexistente;
		
			endcase
		
		
		end
		
		//Carrega Shift
		
		carrega_Shift: begin 
		
		Shift = 3'b001;
		
		case(funct)
					6'h00:  	estado <= sll;

					6'h04:		estado <= sllv;
		
					6'h03: 		estado <= sra;
	
					6'h07:		estado <= srav;

					6'h02:		estado <= srl;
		endcase
		end
		
		
		// DIV 
		
		div: begin
			
			DivIn = 1'b1;
			
			if(DivOut == 1'b1)
				begin
					DivIn = 1'b0;
					Load_Hi = 1'b1;
					Load_Lo = 1'b1;
					estado <= busca_Instrucao;
				end 
			else begin 
				estado <= div;
			end
						
		end

		// ADD

		add: begin

			ALUSourceA = 1'b1;
			ALUSourceB = 3'b000;
			ALUControl = 3'b001;
			estado <= wait_overflow;
		
		end

		// TESTA CONDIO DE OVERFLOW

		wait_overflow: begin	
		
			if(overflow == 1'b1)begin
				estado <= estado_overflow;
			end
			else begin
				estado <= escreve_Rd;
			end
		
		end
		
		wait_overflow2: begin	
		
			if(overflow == 1'b1)begin
				estado <= estado_overflow;
			end
			else begin
				estado <= escreve_Rt;
			end
		
		end

		// ESCREVE EM RD

		escreve_Rd: begin	
		
			WriteData = 3'b001;
			RegDest = 2'b01;
			RegWrite = 1'b1;
		
			estado <= busca_Instrucao;
		
		end
		
		escreve_Rt: begin
			WriteData = 3'b001;
			RegDest = 2'b00;
			RegWrite = 1'b1;
		
			estado <= busca_Instrucao;
		end
		
		// SUB

		sub: begin

			ALUSourceA = 1'b1;
			ALUSourceB = 3'b000;
			ALUControl = 3'b010;
			estado <= wait_overflow;
		
		end


		// OVERFLOW
	
		estado_overflow: begin
		
			MemoryAdress = 2'b10;
			MemoryWriteOrRead = 1'b0;
			ALUSourceA = 1'b0;
			ALUSourceB = 3'b001;
			ALUControl = 3'b010;			

		estado <= escreve_EPC;
		
		end	
		
		//ESCREVE EPC
		
		escreve_EPC: begin
		
			EPCWrite = 1'b1;
		
			estado <= wait5;
		
		end
	
		wait5:
		begin 
		
		estado <= escreve_PC;
		
		end 
		// ESCREVE NO PC

		escreve_PC: begin

			PCWrite = 1'b1;	
			PCSource = 3'b011;
			
			estado <= busca_Instrucao;

		end

		// CALCULO ENDERECO

		calculo_Endereco: begin

			ALUSourceA = 1'b1;
			ALUSourceB = 3'b010;
			ALUControl = 3'b001;
			Load_ALUOut = 1'b1;

			case (opcode)
				6'h20: estado <= ler_memoria_loadb;
				6'h21: estado <= ler_memoria_loadh;
				6'h23: estado <= ler_memoria_loadw;
				6'h28: estado <= ler_memoria_storeb;
				6'h29: estado <= ler_memoria_storeh;
				6'h2b: estado <= storeWord;
			endcase
	
		end

		//  LER MEMORIA LOAD WORD

		ler_memoria_loadw: begin
			
			MemoryWriteOrRead = 1'b0;
			MemoryAdress = 2'b11;
			estado <= wait_memory_loadW;
		end

		//  LER MEMORIA LOAD BYTE

		ler_memoria_loadb: begin
			
			MemoryWriteOrRead = 1'b0;
			MemoryAdress = 2'b11;
			estado <= wait_memory_loadB;
		end

		//  LER MEMORIA LOAD BYTE

		ler_memoria_loadh: begin
			
			MemoryWriteOrRead = 1'b0;
			MemoryAdress = 2'b11;
			estado <= wait_memory_loadH;
		end

		// WAIT MEMORY LOAD WORD

		wait_memory_loadW: begin
						
			estado <= load_MDR_lw;

		end

		
		// WAIT MEMORY LOAD BYTE

		wait_memory_loadB: begin
						
			estado <= load_MDR_lb;

		end
		
		// WAIT MEMORY LOAD HALF

		wait_memory_loadH: begin
			
			estado <= load_MDR_lh;

		end

		// LOAD MDR LOAD WORD
		
		load_MDR_lw: begin
				
			MDRLoad = 1'b1;	
			estado <= loadWord;
		end
			
		// LOAD MDR LOAD HALF
		
		load_MDR_lh: begin
			
			MDRLoad = 1'b1;
			estado <= loadHalf;
		
		end
		
		

		// LOAD MDR LOAD BYTE
		
		load_MDR_lb: begin
			
			MDRLoad = 1'b1;
			estado <= loadByte;
		
		end

		
		// LER MEMORIA STORE HALF
		
		ler_memoria_storeh: begin
			
			MemoryWriteOrRead = 1'b0;
			MemoryAdress = 2'b11;
			estado <= wait_Memory_StoreH;
		
		end
		
		// LER MEMORIA STORE BYTE
		
		ler_memoria_storeb: begin
			
			MemoryWriteOrRead = 1'b0;
			MemoryAdress = 2'b11;
			estado <= wait_Memory_StoreB;
		
		end
		
				
		// STORE WORD

		storeWord: begin

			StoreW = 1'b1;		
			MemoryAdress = 2'b11;
			MemoryWriteOrRead = 1'b1;
			estado <= wait_Store;

		end

		// WAIT MEMORY STORE HALF

		wait_Memory_StoreH: begin
		
			estado <= load_MDR_storeh;
		
		end
		
		// WAIT MEMORY STORE BYTE
		
		wait_Memory_StoreB: begin
		
			estado <= load_MDR_storeb;
		
		end

		// LOAD MDR STORE BYTE
	
		load_MDR_storeb: begin

			MDRLoad = 1'b1;
			estado <= storeByte;

		end
	
		// LOAD MDR STORE HALF
		
		load_MDR_storeh: begin

			MDRLoad = 1'b1;
			estado <= storeHalf;
	
		end
		
		// LOAD WORD

		loadWord: begin
	
			WriteData = 3'b011;
			RegDest = 2'b00;
			LoadW = 1'b1;
			
			estado <= busca_Instrucao;

		end

		// LOAD BYTE
		
		loadByte: begin
		
			WriteData = 3'b100;
			RegDest = 2'b00;
			LoadW = 1'b0;
			LoadHorB = 1'b1;
			RegWrite = 1'b1;

			estado <= busca_Instrucao;

		end

		// LOAD HALF
			
		loadHalf: begin
					
			WriteData = 3'b011;
			RegDest = 2'b00;
			LoadW = 1'b0;
			LoadHorB = 1'b0;

			estado <= busca_Instrucao;
		
		end

		// STORE HALF
				
		storeHalf: begin
		
			StoreW = 1'b0;
			MemoryWriteOrRead = 1'b1;
			StoreHorB = 1'b1;

			estado <= wait_Store;
		
		end

		// STORE BYTE
		
		storeByte: begin
		
			StoreW = 1'b0;
			StoreHorB = 1'b0;
			MemoryWriteOrRead = 1'b1;

			estado <= wait_Store;

		end
		
		// WAIT MEMORY STORE
		
		wait_Store: begin
	
			estado <= busca_Instrucao;
			
		end
		
		
		// JR

		jr: begin

			PCWrite = 1'b1;
			PCSource = 3'b010;
	
			estado <= busca_Instrucao;
			
		end
		
		// J
		
		j: begin
		
			PCWrite = 1'b1;
			PCSource = 3'b000;

			estado <= busca_Instrucao;
	
		end
		
		// MFLO
	
		mflo: begin
		
			RegDest = 2'b01;
			HiOrLo = 1'b0;
			WriteData = 3'b101;
			RegWrite = 1'b1;

			estado <= busca_Instrucao;

		end
		
		// MFHI

		mfhi: begin
		
			RegDest = 2'b01;
			HiOrLo = 1'b1;
			WriteData = 3'b101;
			RegWrite = 1'b1;
			
			estado <= busca_Instrucao;
				
		end
		
		// LUI
		
		lui: begin
		
			RegDest = 2'b00;
			WriteData = 3'b000;
			RegWrite = 1'b1;

			estado <= busca_Instrucao;
	
		end
		
		// BREAK

		breaks: begin
		
			ALUSourceA = 1'b0;
			ALUSourceB = 3'b001;
			ALUControl = 3'b010;
			PCSource = 3'b100;		
			PCWrite = 1'b1;

			estado <= busca_Instrucao;	
	
		end
		
		// SLT
	
		slt: begin
		
			ALUSourceB = 3'b000;
			ALUSourceA = 1'b1;
			ALUControl = 3'b111;
			RegDest = 2'b01;
			WriteData = 3'b010;
			RegWrite = 1'b1;

			estado <= busca_Instrucao;
		
		end
		
		// ADDIU
		
		addiu: begin
		
			ALUSourceB = 3'b010;
			ALUSourceA = 1'b1;
			ALUControl = 3'b001;
			
			estado <= escreve_Rt;

		end
		
		// SLTI

		slti: begin	
		
			ALUSourceB = 3'b010;
			ALUSourceA = 1'b1;
			ALUControl = 3'b111;
			RegDest = 2'b00;
			WriteData = 3'b010;
			RegWrite = 1'b1;
				
			estado <= busca_Instrucao;
		
		end
		
		// ADDI

		addi: begin
		
			ALUSourceB = 3'b010;
			ALUSourceA = 1'b1;
			ALUControl = 3'b001;
		
			estado <= wait_overflow2;

		end
		
		// OPCODE INEXISTENTE

		opcode_inexistente: begin	
		
			MemoryAdress = 2'b01;
			MemoryWriteOrRead = 1'b1;
			ALUSourceA = 1'b0;
			ALUSourceB = 3'b001;
			ALUControl = 3'b010;

			estado <= escreve_EPC ;

		end
		
		
		
		// RTE 
		
		rte: begin 
		
			PCSource = 3'b101;
			PCWrite = 1'b1;
 
			estado <= busca_Instrucao;
	
		end		
				
		// JAL
		
		jal: begin
		
			PCSource = 3'b000;
			PCWrite = 1'b1;

			estado <= busca_Instrucao;	
		
		end
		
		
		// ESCREVE EM RA
		
		escreve_Ra: begin
		
		
			RegDest = 2'b11;
			RegWrite = 1'b1;
			WriteData = 3'b111;
		
			estado <= jal;
						
		end
		
		// SLL
		
		sll: begin

			
			Shift_ShamtOrRt = 1'b1;		
			Shift = 3'b010;		
			
			estado <= carrega_shiftout;
			
		end
		
		// SLLV

		sllv: begin
		
			
			Shift_ShamtOrRt = 1'b0;		
			Shift = 3'b010;		
			
			estado <= carrega_shiftout;
		
		end
		
		// SRA

		sra: begin
		
			
			Shift_ShamtOrRt = 1'b1;		
			Shift = 3'b100;
			
			estado <= carrega_shiftout;
		
		end
		
		// SRAV

		srav: begin

			
			Shift_ShamtOrRt = 1'b0;		
			Shift = 3'b100;
			
			estado <= carrega_shiftout;
			
		end
		
		// SRL

		srl: begin

			
			Shift_ShamtOrRt = 1'b1;		
			Shift = 3'b011;
			
			estado <= carrega_shiftout;
		
		end
		
		//Carrega shift
		carrega_shiftout: begin
			Load_shift_Out = 1'b1;
			estado <= escreve_Shift;
			
		
		end
		// ESCREVE SHIFT

		escreve_Shift: begin
		
			WriteData = 3'b011;
			RegDest = 2'b01;
			RegWrite = 1'b1;
			estado <= busca_Instrucao;
		
		end
		
		// ESTADO COMPARA
		
		estado_compara: begin
		
			ALUSourceA = 1'b1;
			ALUSourceB = 3'b000;  
			ALUControl = 3'b111;
		
			if(opcode == 6'h4) begin
				estado <= beq;         
			end else begin
				estado <= bne;
			end

		end
		
		// BNE

		bne: begin

			BneOrBeq = 1'b1;
			PCSource = 3'b001;
		
			estado <= busca_Instrucao;
		
		end

		// BEQ
		
		beq: begin

			BneOrBeq = 1'b0;
			PCSource = 3'b001;
		
			estado <= busca_Instrucao;

		end
		
		// AND
		
		estado_and: begin
	
			ALUSourceA = 1'b1;
			ALUSourceB = 3'b000;
			ALUControl = 3'b011;
		
			estado <= escreve_Rd;
		
		end

	endcase
end
endmodule		